/*------------------------------------------------------------------
  lm78.h

  A Linux driver for National Semiconductor LM78 Microprocessor System
  Hardware Monitor.  (data sheet available from http://www.national.com)

  Erik Hendriks
  hendriks@cesdis.gsfc.nasa.gov

  $Id: lm78.h,v 1.3 1998/07/30 17:47:33 hendriks Exp $
  ------------------------------------------------------------------*/
#ifndef _LM78_H_
#define _LM78_H_

/*--------------------------------------------------------------------
 * Known hardware monitor types
 */
#define LM78    1
#define LM78J   2
#define W83781D 3

/*--------------------------------------------------------------------
 * Library to use the LM78 Microprocessor Hardware Monitor
 */
#define LM78_DEF_BASE 0x290
extern int LM78_BASE;
#define LM78_ADDR (LM78_BASE+0x5)
#define LM78_DATA (LM78_BASE+0x6)

/*--- Control Registers ------------------------*/
#define LM78_CONFIG       0x40
/*--- Bits for the config register ---*/
#define LM78_CONF_START   0x01
#define LM78_CONF_SMI     0x02
#define LM78_CONF_NMI     0x04
#define LM78_CONF_INT     0x08
#define LM78_CONF_RESET   0x10
#define LM78_CONF_IRQ     0x20
#define LM78_CONF_POWER   0x40

#define LM78_CONF_INIT    0x80
#define WB_CONF_INIT      0x01

/*--- Interrupt status registers ---*/
#define LM78_DATA_INT1    0x41
#define LM78_DATA_INT2    0x42

/*--- Interrupt mask registers (for SMI) ---*/
#define LM78_CONF_SMI1    0x43
#define LM78_CONF_SMI2    0x44
/*--- Interrupt mask registers (for NMI/IRQ) ---*/
#define LM78_CONF_NMI1    0x45
#define LM78_CONF_NMI2    0x46

#define LM78_CONF_VID_FAN  0x47
#define LM78_SERIAL_ADDR   0x48
#define LM78_CHIP_ID       0x49

/*--- These are specific to the Winbond W83781D ---*/
#define WB_BANK_SEL        0x4E
#define WB_VENDOR_ID       0x4F
#define WB_BEEP_CTRL       0x0056
#define WB_CHIP_ID         0x0058

#define LM78_DATA_POST_RAM 0x00
#define LM78_DATA_VAL_RAM  0x20
#define LM78_DATA_VAL_RAM2 0x60

/*--- Data Registers ---------------------------*/
#define LM78_DATA_IN0     0x20
#define LM78_DATA_IN1     0x21
#define LM78_DATA_IN2     0x22
#define LM78_DATA_IN3     0x23
#define LM78_DATA_IN4     0x24
#define LM78_DATA_IN5     0x25
#define LM78_DATA_IN6     0x26
#define LM78_DATA_TEMP    0x27
#define LM78_DATA_FAN1    0x28
#define LM78_DATA_FAN2    0x29
#define LM78_DATA_FAN3    0x2A

/* The W83781D has a notion of register banks.  To
 * denote which bank a register exists in, I'm sticking
 * the bank number in bits 8-15 of the register name */
#define WB_DATA_TEMP_2       0x0150
#define WB_DATA_TEMP_3       0x0250

/*--- Limit Registers --------------------------*/
/* For the IN's +0 = high limit                 */
/*              +1 = low limit                  */
#define LM78_LIM_IN0      0x2B
#define LM78_LIM_IN1      0x2D
#define LM78_LIM_IN2      0x2F
#define LM78_LIM_IN3      0x31
#define LM78_LIM_IN4      0x33
#define LM78_LIM_IN5      0x35
#define LM78_LIM_IN6      0x37
#define LM78_LIM_OVERTEMP 0x39
#define LM78_LIM_TEMPHYST 0x3A
#define LM78_LIM_FAN1     0x3B
#define LM78_LIM_FAN2     0x3C
#define LM78_LIM_FAN3     0x3D

#define WB_LIM_HYST_2     0x153
#define WB_LIM_OVT_2      0x155
#define WB_LIM_HYST_3     0x253
#define WB_LIM_OVT_3      0x255

/*--- Fake "registers" for the proc/sysctl interface ---*/
#define LM78_DATA_CHASSIS    0x10000
#define LM78_CONTROL_RESET   0x10001
#define LM78_DATA_INTERRUPTS 0x10002

/*--- Interrupt masks --------------------------*/
#define LM78_INT_IN0      0x0001
#define LM78_INT_IN1      0x0002
#define LM78_INT_IN2      0x0004
#define LM78_INT_IN3      0x0008
#define LM78_INT_TEMP     0x0010
#define LM78_INT_BTI      0x0020
#define LM78_INT_FAN1     0x0040
#define LM78_INT_FAN2     0x0080
#define LM78_INT_IN4      0x0100
#define LM78_INT_IN5      0x0200
#define LM78_INT_IN6      0x0400
#define LM78_INT_FAN3     0x0800
#define LM78_INT_CHASSIS  0x1000
#define LM78_INT_FIFO     0x2000
#define LM78_INT_SMI_IN   0x4000
#define LM78_INT_RESRVD   0x8000

#define WB_INT_TEMP23     0x0020

static inline void
LM78_select(unsigned regno) {
    /* Make sure the serial interface isn't busy. */
    /* This would be good but we still need a timeout mechanism since
     * we might not really be looking at an LM78 */
    /*while (inb(LM78_ADDR) & 0x80);*/
    outb(regno, LM78_ADDR);
}

static inline unsigned char
LM78_get(unsigned regno) {
    LM78_select(regno);
    return inb(LM78_DATA);
}

static inline void
LM78_set(unsigned regno, unsigned char value) {
    LM78_select(regno);
    outb(value, LM78_DATA);
}

static inline void
LM78_or_reg(unsigned regno, unsigned char value) {
    unsigned char c;
    c = LM78_get(regno);
    c |= value;
    LM78_set(regno, c);
}

static inline void
LM78_and_reg(unsigned regno, unsigned char value) {
    unsigned char c;
    c = LM78_get(regno);
    c &= value;
    LM78_set(regno, c);
}

/*-- Winbond Specific functions Register access functions ---*/
static inline void
WB_bank_select(int bank) {
    unsigned char b;
    b = LM78_get(WB_BANK_SEL);
    /* Side effect here: turn off high byte access bit */
    b &= ~0x87;
    b |= bank;
    LM78_set(WB_BANK_SEL, b);
}

/* These get/set functions will also do bank selection if necessary. */
static inline unsigned char
WB_get(int reg) {
    unsigned char val;
    if (reg & 0xFF00) {
	WB_bank_select(reg>>8);
	val = LM78_get(reg&0xFF);
	WB_bank_select(0);
    } else
	val = LM78_get(reg&0xFF);
    return val;
}

static inline void
WB_set(int reg, unsigned char value) {
    if (reg & 0xFF00) {
	WB_bank_select(reg>>8);
	LM78_set(reg&0xFF, value);
	WB_bank_select(0);
    } else
	LM78_set(reg&0xFF, value);
}

static inline short
WB_read_short(int reg) {
    short val;
    LM78_and_reg(WB_BANK_SEL, 0x7F); /* Select low bit */
    val = LM78_get(reg);
    LM78_or_reg(WB_BANK_SEL, 0x80); /* Select high bit */
    val |= ((short)LM78_get(reg)) << 8;
    LM78_and_reg(WB_BANK_SEL, 0x7F); /* Select low bit */
    return val;
}

#endif /* lm78.h included */
